using System;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using System.Collections;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Windows.Forms;

namespace WindowsApplication3
{
/// <summary>
/// Zusammenfassung fr MeshGenerator.
/// </summary>
public class MeshGenerator : System.Windows.Forms.Form
{
    byte [,,] meshRawData;
    int cubeLenght;
    int cubeBorder;
    ArrayList vertices;
    ArrayList indices;
    Device meshDevice;

    Mesh surfaceMesh;

    public MeshGenerator (Form windows)
    {
        PresentParameters presentParams = new PresentParameters();
        presentParams.Windowed=true;
        presentParams.SwapEffect=SwapEffect.Discard;
        meshDevice=new Device(0, DeviceType.Hardware, this, CreateFlags.HardwareVertexProcessing, presentParams);

        vertices=new ArrayList();
        indices=new ArrayList();
    }

    public void GenerateMesh (int frameCounter)
    {
        for(int frameNumber = 0; frameNumber<=frameCounter; frameNumber++)
        {
            InitializeRawData(frameNumber);
            GenerateMesh();
            WriteData(frameNumber);

            //Cleanup before next Frame is calculated
            vertices.Clear();
            indices.Clear();
            meshRawData=null;
        }
    }

    public void InitializeRawData (int frameNumber)
    {
        FileStream s = new FileStream("frame"+frameNumber+".ser", FileMode.Open);
        IFormatter f = new BinaryFormatter();

        meshRawData=f.Deserialize(s)as byte [,,];
        s.Close();
        cubeLenght=meshRawData.GetLength(0);
        cubeBorder=cubeLenght-1;
    }

    public void GenerateMesh ()
    {
        int indicesCounter = 0;
        byte previousCube = 0;
        byte actualCube = 0;

        // Front to Back (Z)
        for(int x = 0; x<cubeLenght; x++)
        {
            for(int y = 0; y<cubeLenght; y++)
            {
                for(int z = 0; z<cubeLenght; z++)
                {
                    if(x==cubeBorder || y==cubeBorder || z==cubeBorder)
                    {
                        actualCube=0;
                    }
                    else
                    {
                        actualCube=meshRawData[x, y, z];
                    }

                    if(previousCube==0)
                    {
                        if(actualCube>0)
                        {
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x+1, y+1, z),
                                                                          System.Drawing.Color.Yellow.ToArgb()));
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x, y+1, z), 0));
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x, y, z), 0));
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x+1, y, z), 0));

                            indices.Add(indicesCounter);
                            indices.Add(indicesCounter+1);
                            indices.Add(indicesCounter+2);
                            indices.Add(indicesCounter);
                            indices.Add(indicesCounter+2);
                            indices.Add(indicesCounter+3);
                            indicesCounter+=4;
                        }
                    }

                    if(actualCube==0)
                    {
                        if(previousCube>0)
                        {
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x, y+1, z),
                                                                          System.Drawing.Color.Blue.ToArgb()));
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x+1, y+1, z), 0));
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x+1, y, z), 0));
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x, y, z), 0));

                            indices.Add(indicesCounter);
                            indices.Add(indicesCounter+1);
                            indices.Add(indicesCounter+2);
                            indices.Add(indicesCounter);
                            indices.Add(indicesCounter+2);
                            indices.Add(indicesCounter+3);
                            indicesCounter+=4;
                        }
                    }

                    previousCube=actualCube;
                }
            }
        }

        // Bottom to Top (Y)
        for(int x = 0; x<cubeLenght; x++)
        {
            for(int z = 0; z<cubeLenght; z++)
            {
                for(int y = 0; y<cubeLenght; y++)
                {
                    if(x==cubeBorder || y==cubeBorder || z==cubeBorder)
                    {
                        actualCube=0;
                    }
                    else
                    {
                        actualCube=meshRawData[x, y, z];
                    }

                    if(previousCube==0)
                    {
                        if(actualCube>0)
                        {
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x+1, y, z),
                                                                          System.Drawing.Color.Green.ToArgb()));
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x, y, z), 0));
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x, y, z+1), 0));
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x+1, y, z+1), 0));

                            indices.Add(indicesCounter);
                            indices.Add(indicesCounter+1);
                            indices.Add(indicesCounter+2);
                            indices.Add(indicesCounter);
                            indices.Add(indicesCounter+2);
                            indices.Add(indicesCounter+3);
                            indicesCounter+=4;
                        }
                    }

                    if(actualCube==0)
                    {
                        if(previousCube>0)
                        {
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x+1, y, z+1),
                                                                          System.Drawing.Color.Red.ToArgb()));
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x, y, z+1), 0));
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x, y, z), 0));
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x+1, y, z), 0));

                            indices.Add(indicesCounter);
                            indices.Add(indicesCounter+1);
                            indices.Add(indicesCounter+2);
                            indices.Add(indicesCounter);
                            indices.Add(indicesCounter+2);
                            indices.Add(indicesCounter+3);
                            indicesCounter+=4;
                        }
                    }

                    previousCube=actualCube;
                }
            }
        }

        // Left to Right (X)
        for(int z = 0; z<cubeLenght; z++)
        {
            for(int y = 0; y<cubeLenght; y++)
            {
                for(int x = 0; x<cubeLenght; x++)
                {
                    if(x==cubeBorder || y==cubeBorder || z==cubeBorder)
                    {
                        actualCube=0;
                    }
                    else
                    {
                        actualCube=meshRawData[x, y, z];
                    }

                    if(previousCube==0)
                    {
                        if(actualCube>0)
                        {
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x, y, z),
                                                                          System.Drawing.Color.Gray.ToArgb()));
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x, y+1, z), 0));
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x, y+1, z+1), 0));
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x, y, z+1), 0));

                            indices.Add(indicesCounter);
                            indices.Add(indicesCounter+1);
                            indices.Add(indicesCounter+2);
                            indices.Add(indicesCounter);
                            indices.Add(indicesCounter+2);
                            indices.Add(indicesCounter+3);
                            indicesCounter+=4;
                        }
                    }

                    if(actualCube==0)
                    {
                        if(previousCube>0)
                        {
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x, y, z+1),
                                                                          System.Drawing.Color.BlueViolet.ToArgb()));
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x, y+1, z+1), 0));
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x, y+1, z), 0));
                            vertices.Add(new CustomVertex.PositionColored(new Vector3(x, y, z), 0));

                            indices.Add(indicesCounter);
                            indices.Add(indicesCounter+1);
                            indices.Add(indicesCounter+2);
                            indices.Add(indicesCounter);
                            indices.Add(indicesCounter+2);
                            indices.Add(indicesCounter+3);
                            indicesCounter+=4;
                        }
                    }

                    previousCube=actualCube;
                }
            }
        }
    }

    public void WriteData (int frameNumber)
    {
        if(surfaceMesh!=null)
            surfaceMesh.Dispose();

        surfaceMesh=new Mesh(vertices.Count/2,                               // 12 faces
                             vertices.Count,                                 // 8 vertices
                             MeshFlags.Use32Bit,                             // no flags
                             VertexFormats.Position | VertexFormats.Diffuse, // Position information only
                             meshDevice);

        surfaceMesh.SetVertexBufferData(vertices.ToArray(typeof(CustomVertex.PositionColored)), LockFlags.None);
        surfaceMesh.SetIndexBufferData(indices.ToArray(typeof(int)), LockFlags.None);
        int []adjacency = new int[surfaceMesh.NumberFaces*3];
        surfaceMesh.GenerateAdjacency(0, adjacency);

        /*
            surfacemesh.OptimizeInPlace(MeshFlags.OptimizeVertexCache | MeshFlags.OptimizeAttrSort | MeshFlags.OptimizeVertexCache, adjacency);
            int []adjacencyopt = new int[surfacemesh.NumberFaces * 3];
            surfacemesh.GenerateAdjacency(0.01f, adjacencyopt);
            //simplifiedMesh = Mesh.Simplify(meshgenerated, adjacencyopt, null, 6, MeshFlags.SimplifyFace);
        */

        ExtendedMaterial []material = new ExtendedMaterial[0];
        surfaceMesh.Save(@"frame"+frameNumber+".x", adjacency, material, XFileFormat.Binary);
    }
}
}
